@@ -10,7 +10,7 @@
 #' train <- agaricus.train
 #' test <- agaricus.test
 #' bst <- xgboost(data = train$data, label = train$label, max.depth = 2, 
-#'                eta = 1, nround = 2,objective = "binary:logistic")
+#'                eta = 1, nthread = 2, nround = 2,objective = "binary:logistic")
 #' xgb.save(bst, 'xgb.model')
 #' bst <- xgb.load('xgb.model')
 #' pred <- predict(bst, test$data)
@@ -19,5 +19,9 @@
 xgb.load <- function(modelfile) {
   if (is.null(modelfile)) 
     stop("xgb.load: modelfile cannot be NULL")
-  xgb.Booster(modelfile = modelfile)
+  
+  handle <- xgb.Booster(modelfile = modelfile)
+  bst <- xgb.handleToBooster(handle)
+  bst <- xgb.Booster.check(bst)
+  return(bst)
 } 
