@@ -1,12 +1,11 @@
 #ifndef XGBOOST_WRAPPER_H_
 #define XGBOOST_WRAPPER_H_
 /*!
- * \file xgboost_wrapperh
+ * \file xgboost_wrapper.h
  * \author Tianqi Chen
  * \brief a C style wrapper of xgboost
  *  can be used to create wrapper of other languages
  */
-#include <cstdio>
 #ifdef _MSC_VER
 #define XGB_DLL __declspec(dllexport)
 #else
@@ -15,8 +14,9 @@
 // manually define unsign long
 typedef unsigned long bst_ulong;
 
-
+#ifdef __cplusplus
 extern "C" {
+#endif
   /*!
    * \brief load a data matrix 
    * \return a loaded data matrix
@@ -41,7 +41,7 @@ extern "C" {
    * \param col_ptr pointer to col headers
    * \param indices findex
    * \param data fvalue
-   * \param nindptr number of rows in the matix + 1 
+   * \param nindptr number of rows in the matix + 1
    * \param nelem number of nonzero elements in the matrix
    * \return created dmatrix
    */
@@ -178,12 +178,18 @@ extern "C" {
    * \brief make prediction based on dmat
    * \param handle handle
    * \param dmat data matrix
-   * \param output_margin whether only output raw margin value
+   * \param option_mask bit-mask of options taken in prediction, possible values
+   *          0:normal prediction
+   *          1:output margin instead of transformed value
+   *          2:output leaf index of trees instead of leaf value, note leaf index is unique per tree
    * \param ntree_limit limit number of trees used for prediction, this is only valid for boosted trees
    *    when the parameter is set to 0, we will use all the trees
    * \param len used to store length of returning result
    */
-  XGB_DLL const float *XGBoosterPredict(void *handle, void *dmat, int output_margin, unsigned ntree_limit, bst_ulong *len);
+  XGB_DLL const float *XGBoosterPredict(void *handle, void *dmat, 
+                                        int option_mask, 
+                                        unsigned ntree_limit,
+                                        bst_ulong *len);
   /*!
    * \brief load model from existing file
    * \param handle handle
@@ -196,14 +202,32 @@ extern "C" {
    * \param fname file name
    */
   XGB_DLL void XGBoosterSaveModel(const void *handle, const char *fname);
+  /*!
+   * \brief load model from in memory buffer
+   * \param handle handle
+   * \param buf pointer to the buffer
+   * \param len the length of the buffer
+   */
+  XGB_DLL void XGBoosterLoadModelFromBuffer(void *handle, const void *buf, bst_ulong len);
+  /*!
+   * \brief save model into binary raw bytes, return header of the array
+   * user must copy the result out, before next xgboost call
+   * \param handle handle
+   * \param out_len the argument to hold the output length
+   * \return the pointer to the beginning of binary buffer
+   */
+  XGB_DLL const char *XGBoosterGetModelRaw(void *handle, bst_ulong *out_len);
   /*!
    * \brief dump model, return array of strings representing model dump
    * \param handle handle
    * \param fmap  name to fmap can be empty string
+   * \param with_stats whether to dump with statistics
    * \param out_len length of output array
    * \return char *data[], representing dump of each model
    */
   XGB_DLL const char **XGBoosterDumpModel(void *handle, const char *fmap,
-                                          bst_ulong *out_len);
+                                          int with_stats, bst_ulong *out_len);
+#ifdef __cplusplus
 }
+#endif
 #endif  // XGBOOST_WRAPPER_H_
